@@ -4,41 +4,36 @@
 # either version 2, or (at your option) any later version.
 
 
-
 import errno
-import os.path
+import os
 import pdb
-import sys
 import signal
+import ssl
+import sys
 import traceback
-from urlgrabber.grabber import URLGrabError
+from http.client import HTTPException, BadStatusLine
+from urllib.error import URLError, HTTPError
 
-from osc import oscerr
-from .oscsslexcp import NoSecureSSLError
-from osc.util.cpio import CpioError
-from osc.util.packagequery import PackageError
+import urllib3.exceptions
 
-try:
-    from M2Crypto.SSL.Checker import SSLVerificationError
-    from M2Crypto.SSL import SSLError as SSLError
-except:
-    SSLError = None
-    SSLVerificationError = None
+from . import _private
+from . import commandline
+from . import oscerr
+from .OscConfigParser import configparser
+from .oscssl import CertVerificationError
+from .util.cpio import CpioError
+from .util.helper import decode_it
+from .util.packagequery import PackageError
 
 try:
     # import as RPMError because the class "error" is too generic
-    from rpm import error as RPMError #pylint: disable=no-name-in-module
+    # pylint: disable=E0611
+    from rpm import error as RPMError
 except:
     # if rpm-python isn't installed (we might be on a debian system):
-    RPMError = None
+    class RPMError(Exception):
+        pass
 
-try:
-    from http.client import HTTPException, BadStatusLine
-    from urllib.error import URLError, HTTPError
-except ImportError:
-    #python 2.x
-    from http.client import HTTPException, BadStatusLine
-    from urllib.error import URLError, HTTPError
 
 # the good things are stolen from Matt Mackall's mercurial
 
@@ -46,6 +41,8 @@ except ImportError:
 def catchterm(*args):
     raise oscerr.SignalInterrupt
 
+
+# Signals which should terminate the program safely
 for name in 'SIGBREAK', 'SIGHUP', 'SIGTERM':
     num = getattr(signal, name, None)
     if num:
@@ -75,23 +72,18 @@ def run(prg, argv=None):
             raise
     except oscerr.SignalInterrupt:
         print('killed!', file=sys.stderr)
-        return 1
     except KeyboardInterrupt:
         print('interrupted!', file=sys.stderr)
-        return 1
+        return 130
     except oscerr.UserAbort:
         print('aborted.', file=sys.stderr)
-        return 1
     except oscerr.APIError as e:
         print('BuildService API error:', e.msg, file=sys.stderr)
-        return 1
     except oscerr.LinkExpandError as e:
         print('Link "%s/%s" cannot be expanded:\n' % (e.prj, e.pac), e.msg, file=sys.stderr)
         print('Use "osc repairlink" to fix merge conflicts.\n', file=sys.stderr)
-        return 1
     except oscerr.WorkingCopyWrongVersion as e:
         print(e, file=sys.stderr)
-        return 1
     except oscerr.NoWorkingCopy as e:
         print(e, file=sys.stderr)
         if os.path.isdir('.git'):
@@ -102,7 +94,6 @@ def run(prg, argv=None):
             print("Current directory looks like svn.", file=sys.stderr)
         if os.path.isdir('CVS'):
             print("Current directory looks like cvs.", file=sys.stderr)
-        return 1
     except HTTPError as e:
         print('Server returned an error:', e, file=sys.stderr)
         if hasattr(e, 'osc_msg'):
@@ -119,94 +110,103 @@ def run(prg, argv=None):
             print(body, file=sys.stderr)
 
         if e.code in [400, 403, 404, 500]:
-            if '<summary>' in body:
-                msg = body.split('<summary>')[1]
-                msg = msg.split('</summary>')[0]
-                print(msg, file=sys.stderr)
+            if b'<summary>' in body:
+                msg = body.split(b'<summary>')[1]
+                msg = msg.split(b'</summary>')[0]
+                msg = _private.api.xml_escape(msg)
+                print(decode_it(msg), file=sys.stderr)
         if e.code >= 500 and e.code <= 599:
             print('\nRequest: %s' % e.filename)
             print('Headers:')
-            for h, v in list(e.hdrs.items()):
+            for h, v in e.hdrs.items():
                 if h != 'Set-Cookie':
                     print("%s: %s" % (h, v))
 
-        return 1
     except BadStatusLine as e:
         print('Server returned an invalid response:', e, file=sys.stderr)
         print(e.line, file=sys.stderr)
-        return 1
     except HTTPException as e:
         print(e, file=sys.stderr)
-        return 1
     except URLError as e:
-        print('Failed to reach a server:\n', e.reason, file=sys.stderr)
-        return 1
-    except URLGrabError as e:
-        print('Failed to grab %s: %s' % (e.url, e.exception), file=sys.stderr)
-        return 1
-    except IOError as e:
+        msg = 'Failed to reach a server'
+        if hasattr(e, '_osc_host_port'):
+            msg += ' (%s)' % e._osc_host_port
+        msg += ':\n'
+        print(msg, e.reason, file=sys.stderr)
+    except ssl.SSLError as e:
+        if 'tlsv1' in str(e):
+            print('The python on this system or the server does not support TLSv1.2', file=sys.stderr)
+        print("SSL Error:", e, file=sys.stderr)
+    except OSError as e:
         # ignore broken pipe
         if e.errno != errno.EPIPE:
             raise
-        return 1
     except OSError as e:
         if e.errno != errno.ENOENT:
             raise
         print(e, file=sys.stderr)
-        return 1
     except (oscerr.ConfigError, oscerr.NoConfigfile) as e:
-        print(e.msg, file=sys.stderr)
-        return 1
+        print(e, file=sys.stderr)
+    except configparser.Error as e:
+        print(e.message, file=sys.stderr)
     except oscerr.OscIOError as e:
         print(e.msg, file=sys.stderr)
         if getattr(prg.options, 'debug', None) or \
            getattr(prg.conf, 'config', {}).get('debug', None):
             print(e.e, file=sys.stderr)
-        return 1
     except (oscerr.WrongOptions, oscerr.WrongArgs) as e:
         print(e, file=sys.stderr)
         return 2
     except oscerr.ExtRuntimeError as e:
         print(e.file + ':', e.msg, file=sys.stderr)
-        return 1
     except oscerr.ServiceRuntimeError as e:
         print(e.msg, file=sys.stderr)
     except oscerr.WorkingCopyOutdated as e:
         print(e, file=sys.stderr)
-        return 1
     except (oscerr.PackageExists, oscerr.PackageMissing, oscerr.WorkingCopyInconsistent) as e:
         print(e.msg, file=sys.stderr)
-        return 1
     except oscerr.PackageInternalError as e:
-        print('a package internal error occured\n' \
-            'please file a bug and attach your current package working copy ' \
-            'and the following traceback to it:', file=sys.stderr)
+        print('a package internal error occured\n'
+              'please file a bug and attach your current package working copy '
+              'and the following traceback to it:', file=sys.stderr)
         print(e.msg, file=sys.stderr)
         traceback.print_exc(file=sys.stderr)
-        return 1
     except oscerr.PackageError as e:
         print(e.msg, file=sys.stderr)
-        return 1
     except PackageError as e:
         print('%s:' % e.fname, e.msg, file=sys.stderr)
-        return 1
     except RPMError as e:
         print(e, file=sys.stderr)
-        return 1
-    except SSLError as e:
-        print("SSL Error:", e, file=sys.stderr)
-        return 1
-    except SSLVerificationError as e:
-        print("Certificate Verification Error:", e, file=sys.stderr)
-        return 1
-    except NoSecureSSLError as e:
+    except CertVerificationError as e:
         print(e, file=sys.stderr)
-        return 1
+    except urllib3.exceptions.MaxRetryError as e:
+        print(e.reason, file=sys.stderr)
     except CpioError as e:
         print(e, file=sys.stderr)
-        return 1
     except oscerr.OscBaseError as e:
         print('*** Error:', e, file=sys.stderr)
-        return 1
+    return 1
+
+
+def main():
+    # avoid buffering output on pipes (bnc#930137) Basically,
+    # a "print('foo')" call is translated to a corresponding
+    # fwrite call that writes to the stdout stream (cf.
+    # string_print (Objects/stringobject.c) and builtin_print
+    # (Python/bltinmodule.c)); If no pipe is used, stdout is
+    # a tty/refers to a terminal => the stream is line buffered
+    # (see _IO_file_doallocate (libio/filedoalloc.c)). If a pipe
+    # is used, stdout does not refer to a terminal anymore => the
+    # stream is fully buffered by default (see
+    # _IO_file_doallocate). The following fdopen call makes
+    # stdout line buffered again (at least on systems that
+    # support setvbuf - if setvbuf is not supported, the stream
+    # remains fully buffered (see PyFile_SetBufSize
+    # (Objects/fileobject.c))).
+    if not os.isatty(sys.stdout.fileno()):
+        sys.stdout = os.fdopen(sys.stdout.fileno(), sys.stdout.mode, 1)
+        sys.stderr = os.fdopen(sys.stderr.fileno(), sys.stderr.mode, 1)
+
+    sys.exit(run(commandline.Osc()))
 
 # vim: sw=4 et
