@@ -3,23 +3,17 @@
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
 
-import sys
-import os.path
+
+import os
+from urllib.request import HTTPError
+from urllib.parse import urlparse
+from urllib.parse import unquote
+from urllib.error import URLError
+
 from .core import streamfile
 
-try:
-    from urllib.request import HTTPError
-    from urllib.parse import urlparse
-    from urllib.parse import unquote
-    from urllib.error import URLError
-except ImportError:
-    from urllib2 import HTTPError
-    from urlparse import urlparse
-    from urllib import unquote
-    from urllib2 import URLError
 
-
-class OscFileGrabber(object):
+class OscFileGrabber:
     def __init__(self, progress_obj=None):
         self.progress_obj = progress_obj
 
@@ -33,7 +27,7 @@ class OscFileGrabber(object):
                 f.write(i)
 
 
-class OscMirrorGroup(object):
+class OscMirrorGroup:
     def __init__(self, grabber, mirrors):
         self._grabber = grabber
         self._mirrors = mirrors
